Index: src/Fl_Native_File_Chooser_MAC.mm
===================================================================
--- src/Fl_Native_File_Chooser_MAC.mm	(revision 12466)
+++ src/Fl_Native_File_Chooser_MAC.mm	(working copy)
@@ -602,9 +602,29 @@
   }
   if (_directory && !dir) dir = [[NSString alloc] initWithUTF8String:_directory];
   if (fl_mac_os_version >= 100600) {
-    if (dir) [(NSSavePanel*)_panel performSelector:@selector(setDirectoryURL:) withObject:[NSURL fileURLWithPath:dir]];
-    if (fname) [(NSSavePanel*)_panel performSelector:@selector(setNameFieldStringValue:) withObject:fname];
+    NSString *path = nil;
+    if (dir && fname) {
+      // STR #3406: If both dir + fname specified, combine and pass to setDirectoryURL
+      path = [[NSString alloc] initWithFormat:@"%@/%@", dir, fname];	// dir+fname -> path
+      // See if full path to file exists
+      //    If dir exists but fname doesn't, avoid using setDirectoryURL,
+      //    otherwise NSSavePanel falls back to showing user's Documents dir.
+      //
+      if ( [[NSFileManager defaultManager] fileExistsAtPath:path] ) {
+	// Set only if full path exists
+        [(NSSavePanel*)_panel setDirectoryURL:[NSURL fileURLWithPath:path]];
+      } else {
+        // Otherwise, fall back to old behavior setting dir + fname separately..
+        [path release];		// release alloc, and..
+        path = nil;		// ..flag to rest of code we /didn't/ use setDirectoryURL
+      }
+    }
+    if ( !path ) {		// didn't setDirectoryURL to full path? Set dir + fname separately..
+      if (dir) [(NSSavePanel*)_panel performSelector:@selector(setDirectoryURL:) withObject:[NSURL fileURLWithPath:dir]];
+      if (fname) [(NSSavePanel*)_panel performSelector:@selector(setNameFieldStringValue:) withObject:fname];
+    }
     retval = [(NSSavePanel*)_panel runModal];
+    [path release];
   }
   else {
     retval = [(id)_panel runModalForDirectory:dir file:fname];
